@@ -1,7 +1,7 @@
 if {![package vsatisfies [package provide Tcl] 8.5]} return
 if {[info sharedlibextension] != ".dll"} return
 if {[::tcl::pkgconfig get debug]} {
-    package ifneeded dde 1.4.2 [list load [file join $dir tcldde14g.dll] dde]
+    package ifneeded dde 1.4.3 [list load [file join $dir tcldde14g.dll] Dde]
 } else {
-    package ifneeded dde 1.4.2 [list load [file join $dir tcldde14.dll] dde]
+    package ifneeded dde 1.4.3 [list load [file join $dir tcldde14.dll] Dde]
 }
